#!/bin/bash

if [[ $EUID -ne 0 ]]; then
   echo "ERROR: Run this program under root, or sudo -s"
   exit 1
fi

ARCH="arm64"

K3_FULLPATH=`realpath $0`
K3WRK_DIR=`dirname ${K3_FULLPATH}`

echo "Working directory = ${K3WRK_DIR}"

rm -f ${K3WRK_DIR}/astra-mobile-${ARCH}.wic
fallocate -l 16G ${K3WRK_DIR}/astra-mobile-${ARCH}.wic
TARGET_DEVICE="${K3WRK_DIR}/astra-mobile-${ARCH}.wic"

BOOTTAR=`find ${K3WRK_DIR} -type f -name "boot*gz" | head -n 1`
ROOTTAR=`find ${K3WRK_DIR} -type f -name "rootfs*gz" | head -n 1`

echo "Boot archive = ${BOOTTAR}"
echo "Rootfs archive = ${ROOTTAR}"

sync

#dd if=${K3WRK_DIR}/imx-image-multimedia-imx8mnevk.wic of=/dev/mmcblk0p1 conv=fsync

echo "----------: disk partitions creation..."
wipefs -a ${TARGET_DEVICE}
parted -s -a optimal ${TARGET_DEVICE} \
mklabel msdos \
mkpart primary fat16 32MiB  1024MiB \
mkpart primary ext4  1024MiB 100% \
set 1 esp on \
print

sync
sleep 3

echo "loopback setup"
DEST_DEVICE=$(losetup --find --show --partscan ${K3WRK_DIR}/astra-mobile-${ARCH}.wic)
sync

echo "create filesystems"
DEV_PREFIX="p"
mkfs.fat -F32               "${DEST_DEVICE}${DEV_PREFIX}"1
mkfs.ext4 -F  -L "root"     "${DEST_DEVICE}${DEV_PREFIX}"2

#dd if=${K3WRK_DIR}/imx-boot-imx8mnddr4evk-sd.bin-flash_ddr4_evk of=/dev/mmcblk0  bs=1k seek=32 conv=fsync

#dd if=${K3WRK_DIR}/imx-boot-imx8mpddr4evk-sd.bin-flash_ddr4_evk of=${DEST_DEVICE}  bs=1k seek=32 conv=fsync
dd if=${K3WRK_DIR}/imx-boot-imx8mpevk-sd.bin-flash_evk of=${DEST_DEVICE}  bs=1k seek=32 conv=fsync

mkdir -p /mnt/{1,2}
mount "${DEST_DEVICE}${DEV_PREFIX}"1 /mnt/1
mount "${DEST_DEVICE}${DEV_PREFIX}"2 /mnt/2

echo "----------: unpack boot/rootfs to disk..."
tar --xattrs --xattrs-include='*' -zxpf ${BOOTTAR} -C /mnt/1

echo "----------: unpack boot/rootfs to disk..."
tar --xattrs --xattrs-include='*' -zxpf ${ROOTTAR} -C /mnt/2 --strip-components 1

sed -i "s/mmcblk1/mmcblk2/" /mnt/2/etc/fstab

echo "----------- BOOT:"
ls -la /mnt/1

echo "----------- ROOTFS:"
ls -la /mnt/2

sync
umount /mnt/1
umount /mnt/2

losetup -d /dev/loop1
losetup -d /dev/loop0

echo "done"
